@@ -7,7 +7,7 @@
 # http://search.cpan.org/dist/Devel-NYTProf/
 #
 ###########################################################
-# $Id: Data.pm 685 2009-02-15 20:27:13Z tim.bunce $
+# $Id: Data.pm 873 2009-10-22 16:24:17Z tim.bunce $
 ###########################################################
 package Devel::NYTProf::Data;
 
@@ -52,7 +52,7 @@ use Devel::NYTProf::FileInfo;
 use Devel::NYTProf::SubInfo;
 use Devel::NYTProf::Util qw(make_path_strip_editor strip_prefix_from_paths get_abs_paths_alternation_regex);
 
-our $VERSION = '2.08';
+our $VERSION = '2.11';
 
 my $trace = (($ENV{NYTPROF}||'') =~ m/\b trace=(\d+) /x) && $1; # XXX a hack
 
@@ -68,14 +68,16 @@ aggregates the contents, and returns the results as a blessed data structure.
 
 sub new {
     my $class = shift;
-    my $args = shift || { filename => 'nytprof.out' };
+    my $args = shift || { };
 
-    my $file = $args->{filename}
-        or croak "No filename specified";
+    my $file = $args->{filename} ||= 'nytprof.out';
 
     print "Reading $file\n" unless $args->{quiet};
 
-    my $profile = load_profile_data_from_file($file);
+    my $profile = load_profile_data_from_file(
+        $file,
+        $args->{callback},
+    );
     bless $profile => $class;
 
     my $fid_fileinfo = $profile->{fid_fileinfo};
@@ -107,17 +109,19 @@ sub new {
 sub _caches       { return shift->{caches} ||= {} }
 sub _clear_caches { return delete shift->{caches} }
 
+sub attributes {
+    return shift->{attribute} || {};
+}
+
 sub subname_subinfo_map {
     return { %{ shift->{sub_subinfo} } }; # shallow copy
 }
 
-# default:
 # { pkgname => [ subinfo1, subinfo2, ... ], ... }
-# merged:
-# { pkgname => [ single_merged_subinfo ], ...  }
-sub package_subinfo_map {
+# if merged is true then array contains a single 'merged' subinfo
+sub XXXpackage_subinfo_map {
     my $self = shift;
-    my ($merged) = @_;
+    my ($merged_subs, $nested_pkgs) = @_;
 
     my $all_subs = $self->subname_subinfo_map;
     my %pkg;
@@ -125,7 +129,7 @@ sub package_subinfo_map {
         $name =~ s/^(.*::).*/$1/; # XXX $subinfo->package
         push @{ $pkg{$name} }, $subinfo;
     }
-    if ($merged) {
+    if ($merged_subs) {
         while ( my ($pkg_name, $subinfos) = each %pkg ) {
             my $subinfo = shift(@$subinfos)->clone;
             $subinfo->merge_in($_) for @$subinfos;
@@ -136,6 +140,45 @@ sub package_subinfo_map {
     return \%pkg;
 }
 
+# package_tree_subinfo_map is like package_subinfo_map but returns
+# nested data instead of flattened.
+# for "Foo::Bar::Baz" package:
+# { Foo => { '' => [...], '::Bar' => { ''=>[...], '::Baz'=>[...] } } }
+# if merged is true then array contains a single 'merged' subinfo
+sub package_subinfo_map {
+    my $self = shift;
+    my ($merge_subs, $nested_pkgs) = @_;
+
+    my %pkg;
+    my %to_merge;
+
+    my $all_subs = $self->subname_subinfo_map;
+    while ( my ($name, $subinfo) = each %$all_subs ) {
+        $name =~ s/^(.*::).*/$1/; # XXX $subinfo->package
+        my $subinfos;
+        if ($nested_pkgs) {
+            my @parts = split /::/, $name;
+            my $node = $pkg{ shift @parts } ||= {};
+            $node = $node->{ shift @parts } ||= {} while @parts;
+            $subinfos = $node->{''} ||= [];
+        }
+        else {
+            $subinfos = $pkg{$name} ||= [];
+        }
+        push @$subinfos, $subinfo;
+        $to_merge{$subinfos} = $subinfos if $merge_subs;
+    }
+
+    for my $subinfos (values %to_merge) {
+        my $subinfo = shift(@$subinfos)->clone;
+        $subinfo->merge_in($_) for @$subinfos;
+        # replace the many with the one
+        @$subinfos = ($subinfo);
+    }
+
+    return \%pkg;
+}
+
 # [
 #   undef,  # depth 0
 #   {       # depth 1
@@ -284,35 +327,37 @@ sub dump_profile_data {
 
     $self->_clear_caches;
 
-    my $abs_path_regex = $^O eq "MSWin32" ? qr,^\w:/, : qr,^/,;
-    my @abs_inc = grep { $_ =~ $abs_path_regex } $self->inc;
-    my $is_lib_regex = get_abs_paths_alternation_regex(\@abs_inc);
-
     my $callback = sub {
         my ($path, $value) = @_;
 
+        if ($path->[0] eq 'attribute' && @$path == 1) {
+            my %v = %$value;
+            delete @v{qw(PRIVLIB_EXP ARCHLIB_EXP)};
+            return ({}, \%v);
+        }
+
         if ($args->{skip_stdlib}) {
 
             # for fid_fileinfo don't dump internal details of lib modules
             if ($path->[0] eq 'fid_fileinfo' && @$path==2) {
-                my $is_lib = ($value->filename =~ $is_lib_regex) ? 1 : 0;
-                return { skip_internal_details => $is_lib };
+                my $fi = $self->fileinfo_of($value->[0]);
+                return ({ skip_internal_details => $fi->is_perl_std_lib }, $value);
             }
 
             # skip sub_subinfo data for 'library modules'
             if ($path->[0] eq 'sub_subinfo' && @$path==2 && $value->[0]) {
                 my $fi = $self->fileinfo_of($value->[0]);
-                return undef if $fi->filename =~ $is_lib_regex;
+                return undef if $fi->is_perl_std_lib;
             }
 
             # skip fid_*_time data for 'library modules'
             if ($path->[0] =~ /^fid_\w+_time$/ && @$path==2) {
                 my $fi = $self->fileinfo_of($path->[1]);
-                return undef if $fi->filename =~ $is_lib_regex
+                return undef if $fi->is_perl_std_lib
                          or $fi->filename =~ m!^/\.\.\./!;
             }
         }
-        return {};
+        return ({}, $value);
     };
 
     _dump_elements($startnode, $separator, $filehandle, [], $callback);
@@ -352,7 +397,7 @@ sub _dump_elements {
 
         my $dump_opts = {};
         if ($callback) {
-            $dump_opts = $callback->([ @$path, $key ], $value);
+            ($dump_opts, $value) = $callback->([ @$path, $key ], $value);
             next if not $dump_opts;
         }
 
@@ -441,6 +486,7 @@ filenames: eval sequence numbers, like "(re_eval 2)" are changed to 0
 
 sub normalize_variables {
     my $self       = shift;
+    my $attributes = $self->attributes;
 
     for my $attr (qw(
         basetime xs_version perl_version clock_id ticks_per_sec nv_size
@@ -448,14 +494,9 @@ sub normalize_variables {
         total_stmts_duration total_stmts_measured total_stmts_discounted
         total_sub_calls
     )) {
-        $self->{attribute}{$attr} = 0;
+        $attributes->{$attr} = 0;
     }
 
-    my $abs_path_regex = $^O eq "MSWin32" ? qr,^\w:/, : qr,^/,;
-    my @abs_inc = grep { $_ =~ $abs_path_regex } $self->inc;
-    my $is_lib_regex = get_abs_paths_alternation_regex(\@abs_inc);
-
-
     # normalize line data
     for my $level (qw(line block sub)) {
         my $fid_line_data = $self->get_fid_line_data($level) || [];
@@ -557,6 +598,8 @@ sub subs_defined_in_file {
   @subname = $profile->subname_at_file_line($file, $line_number);
   $subname = $profile->subname_at_file_line($file, $line_number);
 
+This method is currently unused and may be deprecated.
+
 =cut
 
 
